#include "xt_int.h"

/*
 * fopen() family
 */
FILE *
fopen(const char *path, const char *mode) {
	FILE *f;
	f = _xt_lib_fopen(path, mode);
	if(_xt_prepare_stdio(f)) {
		fclose(f);
		return NULL;
	}

	/* Initialize fd wrapper */
	_xt_fd_clear(fileno(f), ~0);

	return f;
}

FILE *
fdopen(int fildes, const char *mode) {
	FILE *f;
	f = _xt_lib_fdopen(fildes, mode);
	if(_xt_prepare_stdio(f)) {
		fclose(f);
		return NULL;
	}
	return f;
}

FILE *
freopen(const char *path, const char *mode, FILE *stream) {
	FILE *f;
	f = _xt_lib_freopen(path, mode, stream);
	if(_xt_prepare_stdio(f)) {
		fclose(f);
		return NULL;
	}

	/* Initialize fd wrapper */
	_xt_fd_clear(fileno(f), ~0);

	return f;
}

/*
 * printf() family
 */

int
vfprintf(FILE *stream, const char *format, va_list ap) {
	int ret;

	_xt_fd_lock(fileno(stream), POLLOUT);
	ret = _xt_lib_vfprintf(stream, format, ap);
	_xt_fd_unlock(fileno(stream), POLLOUT);

	return ret;
}


int
vprintf(const char *format, va_list ap) {
	return vfprintf(stdout, format, ap);
}

int
printf(const char *format, ...) {
	va_list ap;
	int ret;

	va_start(ap, format);
	ret = vfprintf(stdout, format, ap);
	va_end(ap);

	return ret;
}

int
fprintf(FILE *stream, const char *format, ...) {
	va_list ap;
	int ret;

	va_start(ap, format);
	ret = vfprintf(stream, format, ap);
	va_end(ap);

	return ret;
}

/*
 * fgets()
 */

char *
fgets(char *str, int size, FILE *stream) {
	char *ret;
	_xt_fd_lock(fileno(stream), POLLIN);
	ret = _xt_lib_fgets(str, size, stream);
	_xt_fd_unlock(fileno(stream), POLLIN);
	return ret;
}

/*
 * fread/fwrite
 */

size_t
fread(void *ptr, size_t size, size_t nmemb, FILE *stream) {
	int ret;
	_xt_fd_lock(fileno(stream), POLLIN);
	ret = _xt_lib_fread(ptr, size, nmemb, stream);
	_xt_fd_unlock(fileno(stream), POLLIN);
	return ret;
}

size_t
fwrite(const void *ptr, size_t size, size_t nmemb, FILE *stream) {
	int ret;
	_xt_fd_lock(fileno(stream), POLLOUT);
	ret = _xt_lib_fwrite(ptr, size, nmemb, stream);
	_xt_fd_unlock(fileno(stream), POLLOUT);
	return ret;
}

