#include "xt_int.h"

/*
 * Create a new thread.
 * start_routine is called with the arg parameter.
 * Thread is destroyed when start_routine is finished.
 */
int
xio_thread_create(xio_thread_t *r_thread, xio_thread_attrs *optThreadAttrs,
	void *(*start_routine)(void *arg), void *arg) {
	xt_thread_t *thread;

	/*
	 * Argument validity checks.
	 */

	if(start_routine == NULL) {
		/*
		 * What you're trying to start?
		 */
		errno = EINVAL;
		return -1;
	}

	if(optThreadAttrs) {
		if(optThreadAttrs->joinable != 0 && r_thread == 0) {
			/*
			 * You're not going to learn thread id
			 * to join it later.
			 */
			errno = EINVAL;
			return -1;
		}
	}

	/*
	 * Create internal thread structure.
	 */

	thread = _xt_new_xio_thread(optThreadAttrs, start_routine, arg, 1);
	if(thread) {
		/*
		 * Report thread identifier.
		 */
		if(r_thread)
			*r_thread = thread;

		return 0;
	}

	/* errno would be set by _xt_new_xt_thread() */
	return -1;
}

/*
 * Wait for specified thread to complete.
 */
int
xio_thread_join(xio_thread_t thread_p, void **value_ptr) {
	xt_thread_t *thread = (xt_thread_t *)thread_p;

	if(thread == NULL) {
		errno = EINVAL;
		return -1;
	}

	if(thread->thread_attrs.joinable == 0) {
		errno = EINVAL;
		return -1;
	}

	if(thread == _xt_current_thread) {
		errno = EDEADLK;
		return -1;
	}

	if(_xt_is_in_ring(thread, _xt_zombie_ring) == 0) {

		/* Wait for the target thread to complete */
		_xt_cond_wait(&thread->join_condition);

	}

	if(value_ptr)
		*value_ptr = thread->return_value;

	/* Move from zombies to the terminated ring */
	_xt_move2ring(thread, &_xt_terminated_ring);

	return 0;
}


