/*
 * Wrapper for the file descriptor.
 */
#ifndef	__XT_FD_H__
#define	__XT_FD_H__

#include "xt_locks.h"

enum xt_fd_flags {
	XFD_INVALID   = 0x01,
	XFD_KQUEUED_R = 0x02,
	XFD_KQUEUED_W = 0x04,
	XFD_READABLE  = 0x10,
	XFD_WRITEABLE = 0x40,
	XFD_HUP	      = 0x100,
};

typedef struct xt_fd_s {
	enum xt_fd_flags fd_flags;
	xt_mutex_t fd_lock[2];	/* Locks on read and write directions */
	xt_thread_t *waiter[2];	/* Thread waiting for I/O */
} xt_fd_t;


/*
 * Lock and unlock file descriptor.
 * poll_direction is either POLLIN or POLLOUT or (POLLIN | POLLOUT).
 * Every function returns 0 in case of success, -1/ENOMEM when
 * failed to create the wrapper for specified fd or -1/EBADF when
 * descriptor number is out of range.
 * Additionally, _xt_fd_trylock returns -1/EBUSY when it fails
 * to lock the descriptor.
 */
int _xt_fd_lock(int fd, int poll_direction);
int _xt_fd_trylock(int fd, int poll_direction);
int _xt_fd_unlock(int fd, int poll_direction);

/*
 * Wait on specified file descriptor
 * Note: descriptor MUST be _xt_fd_lock()'ed in this direction.
 * poll_direction is either POLLIN or POLLOUT, or 0 if thread
 * wants to clear its interest.
 * Returns -1 if something wrong with file descriptor.
 * _xt_fd_wait will place thread into the sleeping ring, but NOT
 * switch the context.
 */
int _xt_fd_wait(int fd, int poll_direction);


/*
 * Set of helper functions.
 */

/* Return the poll()-like state */
int _xt_fd_state(int fd);

/* Set non-blocking I/O on OS-level descriptor */
int _xt_fd_nbio(int fd, int socket_way);

/* Clear specified flags */
void _xt_fd_clear(int fd, enum xt_fd_flags);

/* Clear fd wrapper data. Employs _xt_fd_clear(). */
void _xt_xfd_close(int fd);



/*
 * Kernel queue related staff
 */

extern int _xt_kqueue;

/*
 * Check whether specified file descriptor has installed
 * a kqueue event for the specified interest set.
 */
int _xt_is_queued(int fd, int interest);

/*
 * Gather events optionally change the kevent list and
 * waits for the events to come within tmo_ms.
 * Does NOT involve context switch, just sleeps.
 */
int _xt_gather_events(struct kevent *changelist, int nchanges,
	int tmo_ms);

#endif	/* __XT_FD_H__ */
