#include "xt_int.h"
#include "xt_sched.h"

/*
 * Pointer to the original libc functions.
 */

ssize_t (*_xt_sys_open)(const char *path, int flags, ...) = NULL;
int	(*_xt_sys_close)(int fd) = NULL;
ssize_t (*_xt_sys_read)(int fd, void *buf, size_t nbytes) = NULL;
ssize_t (*_xt_sys_write)(int fd, const void *buf, size_t nbytes) = NULL;
int	(*_xt_sys_socket)(int domain, int type, int protocol) = NULL;
int	(*_xt_sys_accept)(int s, struct sockaddr *addr, socklen_t *addrlen);
int	(*_xt_sys_connect)(int s, const struct sockaddr *name, socklen_t namelen);
int	(*_xt_sys_bind)(int s, const struct sockaddr *addr, socklen_t addrlen);
int	(*_xt_sys_nanosleep)(const struct timespec *rqtp, struct timespec *rmtp);
int	(*_xt_sys_pipe)(int *fildes);

FILE *	(*_xt_lib_fopen)(const char *path, const char *mode);
FILE *	(*_xt_lib_fdopen)(int fildes, const char *mode);
FILE *	(*_xt_lib_freopen)(const char *path, const char *mode, FILE *stream);
int	(*_xt_lib_vfprintf)(FILE *stream, const char *format, va_list);
char *	(*_xt_lib_fgets)(char *str, int size, FILE *stream);
size_t	(*_xt_lib_fread)(void *ptr, size_t size, size_t nmemb, FILE *stream);
size_t	(*_xt_lib_fwrite)(const void *ptr, size_t size, size_t nmemb, FILE *stream);
void	(*_xt_lib_clearerr)(FILE *stream);

void
_init() {
	_xt_sys_open = dlsym(RTLD_NEXT, "open");
	_xt_sys_close = dlsym(RTLD_NEXT, "close");
	_xt_sys_read = dlsym(RTLD_NEXT, "read");
	_xt_sys_write = dlsym(RTLD_NEXT, "write");
	_xt_sys_socket = dlsym(RTLD_NEXT, "socket");
	_xt_sys_bind = dlsym(RTLD_NEXT, "bind");
	_xt_sys_accept = dlsym(RTLD_NEXT, "accept");
	_xt_sys_connect = dlsym(RTLD_NEXT, "connect");
	_xt_sys_nanosleep = dlsym(RTLD_NEXT, "nanosleep");
	_xt_sys_pipe = dlsym(RTLD_NEXT, "pipe");

	_xt_lib_fopen = dlsym(RTLD_NEXT, "fopen");
	_xt_lib_fdopen = dlsym(RTLD_NEXT, "fdopen");
	_xt_lib_freopen = dlsym(RTLD_NEXT, "freopen");
	_xt_lib_vfprintf = dlsym(RTLD_NEXT, "vfprintf");
	_xt_lib_fgets = dlsym(RTLD_NEXT, "fgets");
	_xt_lib_fread = dlsym(RTLD_NEXT, "fread");
	_xt_lib_fwrite = dlsym(RTLD_NEXT, "fwrite");
	_xt_lib_clearerr = dlsym(RTLD_NEXT, "clearerr");

	_xt_kqueue = kqueue();
	if(_xt_kqueue == -1)
		abort();

	if(_xt_timer_initialize() == -1)
		abort();

	/*
	 * Substitute stdio underlying layer with our
	 * thread-aware functions.
	 */
	_xt_prepare_stdio(stdin);
	_xt_prepare_stdio(stdout);
	_xt_prepare_stdio(stderr);

	/*
	 * Initialize current code path as the main thread.
	 */
	_xt_move2ring(_xt_current_thread, &_xt_runnable_ring);

	_xt_idle_thread
		= _xt_new_xio_thread(NULL, _xt_idle_thread_func, NULL, 0);
	if(_xt_idle_thread) {
		_xt_move2ring(_xt_idle_thread, NULL);
	} else {
		abort();
	}

	/* Switch content to myself, saving the last point */
	THR_SWITCH_CONTEXT;
}


