#include "xt_int.h"
#include "xt_locks.h"

/*
 * Mutex support
 */

int
_xt_mutex_trylock(xt_mutex_t *m) {
	if(m->owner == NULL || m->owner == _xt_current_thread) {
		m->owner = _xt_current_thread;
		m->lock_depth++;
		assert(m->lock_depth < 1000);
		return 0;
	} else {
		errno = EBUSY;
		return -1;
	}
}

void
_xt_mutex_lock(xt_mutex_t *m) {

	while(_xt_mutex_trylock(m) != 0) {

		_xt_move2ring(_xt_current_thread, &m->waitq);

		THR_SWITCH_CONTEXT;
	}
}


void
_xt_mutex_unlock(xt_mutex_t *m) {
	if(m->owner != _xt_current_thread)
		return;

	if((--m->lock_depth) == 0) {
		while(m->waitq)	/* Bring all them back to life */
			_xt_move2ring(m->waitq, &_xt_runnable_ring);
		m->owner = NULL;
	}
}

/* Maintainer mode unlock */
void
__xt_mutex_MM_unlock(xt_mutex_t *m) {
	m->lock_depth = 0;

	m->owner = NULL;
	while(m->waitq)
		_xt_move2ring(m->waitq, &_xt_runnable_ring);
}

/*
 * Condition variable support
 */

void
_xt_cond_signal(xt_cond_t *c) {
	if(c->waitq)
		_xt_move2ring(c->waitq, &_xt_runnable_ring);
}

void
_xt_cond_broadcast(xt_cond_t *c) {
	while(c->waitq)
		_xt_move2ring(c->waitq, &_xt_runnable_ring);
}


void
_xt_cond_wait(xt_cond_t *c) {
	_xt_move2ring(_xt_current_thread, &c->waitq);
	THR_SWITCH_CONTEXT;
}

int
_xt_cond_timedwait(xt_cond_t *c, int wait_msec) {

	_xt_schedule_wakeup(wait_msec / 10);

	THR_SWITCH_CONTEXT;

	if(_xt_current_thread->timer.state == XTS_RUNNING) {
		_xt_timer_stop(&_xt_current_thread->timer);
		return 0;
	} else {
		errno = ETIMEDOUT;
		return -1;
	}
}


