#include "xt_int.h"

static xt_thread_t _xt_main_thread;
xt_thread_t *_xt_current_thread = &_xt_main_thread;

void
_xt_thread_t_finish(xt_thread_t *thread) {

	/* Make sure timer is finished */
	_xt_timer_stop(&thread->timer);

	_xt_stack_dispose(thread->stack);

	/* Notify everyone I'm done */
	_xt_cond_broadcast(&thread->join_condition);

	/* Remove thread from the current ring */
	if(thread->thread_attrs.joinable) {
		_xt_move2ring(thread, &_xt_zombie_ring);
	} else {
		_xt_move2ring(thread, &_xt_terminated_ring);
	}

	/* Continue the next thread */
	THR_SWITCH_CONTEXT;
}

static void
_xt_thread_t_fake_main(void) {

	/* Execute thread function. */
	_xt_current_thread->return_value =
		_xt_current_thread->start_routine(_xt_current_thread->arg);

	_xt_thread_t_finish(_xt_current_thread);

	/* Impossible case. */
	abort();
}


xt_thread_t *
_xt_new_xio_thread_body() {
	xt_thread_t *thread;

	thread = _xt_terminated_ring;
	if(thread) {
		/* Remove from the terminated ring */
		_xt_move2ring(thread, NULL);
	} else {
		thread = calloc(1, sizeof(xt_thread_t));
	}

	return thread;
}

xt_thread_t *
_xt_new_xio_thread(xio_thread_attrs *attrs,
		void *(*start_routine)(void *arg),
		void *arg, int run_it) {
	xt_thread_t *thread;
	size_t stack_size = DEFAULT_THREAD_STACK_SIZE;

	if(attrs && attrs->stack_size)
		stack_size = attrs->stack_size;

	thread = _xt_new_xio_thread_body();
	if(thread == NULL)
		return NULL;

	if(attrs)
		memcpy(&thread->thread_attrs, attrs, sizeof(xio_thread_attrs));
	thread->thread_attrs.stack_size = stack_size;	/* Update */

	thread->start_routine = start_routine;
	thread->arg = arg;

	thread->stack = _xt_stack_new(stack_size);
	if(thread->stack == NULL) {
		free(thread);
		return NULL;
	}

	THR_CONTEXT_INIT(thread->context,	
		thread->stack->top,
		_xt_thread_t_fake_main);

	_xt_move2ring(thread, &_xt_runnable_ring);

	if(run_it)
		THR_SWITCH_CONTEXT;

	return thread;
}


