#include "xt_int.h"

xt_thread_t *_xt_runnable_ring = NULL;
xt_thread_t *_xt_sleeping_ring = NULL;
xt_thread_t *_xt_zombie_ring = NULL;
xt_thread_t *_xt_terminated_ring = NULL;


void
_xt_move2ring(xt_thread_t *thread, xt_thread_t **ringp) {
	xt_thread_t *ring;
	xt_thread_t *cur_ring =
		(thread->ring_member.cur_ring)
		? *thread->ring_member.cur_ring
		: (xt_thread_t *)0;

	/*
	 * Remove thread from the current ring.
	 */
	if(cur_ring) {
		if(thread != thread->ring_member.thr_next) {
			thread->ring_member.thr_prev->
				ring_member.thr_next =
					thread->ring_member.thr_next;
			thread->ring_member.thr_next->
				ring_member.thr_prev =
					thread->ring_member.thr_prev;
			/*
			 * Shift ring, if we're at the
			 * current ring position.
			 */
			if(cur_ring == thread) {
				*thread->ring_member.cur_ring
					= thread->ring_member.thr_next;
			}
		} else {
			/* Ring becomes empty. */
			*thread->ring_member.cur_ring = NULL;
		}
	}

	/*
	 * Move to no ring: probably when thread is being destroyed.
	 */
	if(ringp == NULL) {
		thread->ring_member.cur_ring = NULL;
		return;
	}

	ring = *ringp;

	/*
	 * Add thread to the new ring.
	 */
	if(ring) {
		thread->ring_member.thr_prev = ring;
		thread->ring_member.thr_next = ring->ring_member.thr_next;
		ring->ring_member.thr_next->ring_member.thr_prev = thread;
		ring->ring_member.thr_next = thread;
	} else {
		*ringp = thread;
		thread->ring_member.thr_prev = thread;
		thread->ring_member.thr_next = thread;
	}

	thread->ring_member.cur_ring = ringp;
}


int
_xt_is_in_ring(xt_thread_t *thread, xt_thread_t *ring) {
	if(thread->ring_member.cur_ring
		&& (*thread->ring_member.cur_ring == ring))
			return 1;
	return 0;
}


