#include "xt_int.h"

/*
 * Table of unused stacks indexed by their size
 * Last entry in this table is a special treat:
 * it holds too big stacks for some small amount of time
 * just to destroy them later, out of context.
 */
#define	USTACK_TABLE_SIZE	128	/* Up to 512 kbytes on i386 */
static xt_stack_t *unused_stacks[USTACK_TABLE_SIZE];

xt_stack_t *
_xt_stack_new(size_t stack_size) {
	xt_stack_t *stack;
	int stack_cell;

	/* Cap to page size */
	stack_size = (stack_size + (PAGE_SIZE - 1));
	stack_size -= stack_size % PAGE_SIZE;

	stack_cell = stack_size / PAGE_SIZE;
	if(stack_cell < (USTACK_TABLE_SIZE - 1)) {
		stack = unused_stacks[stack_cell];
		if(stack) {
			unused_stacks[stack_cell] = stack->next;
			return stack;
		}
	}

	stack = malloc(sizeof(xt_stack_t));
	if(stack == NULL)
		return NULL;

	stack->base = mmap(NULL, stack_size, PROT_READ | PROT_WRITE,
		MAP_ANON | MAP_NOSYNC | MAP_PRIVATE, -1, 0);
	if(stack->base == MAP_FAILED) {
		free(stack);
		return NULL;
	}

	stack->size = stack_size;
	stack->top = stack->base + stack->size - 4;
	stack->bottom = stack->base;

	return stack;
}

void
_xt_stack_dispose(xt_stack_t *stack) {
	int stack_cell;

	/*
	 * Compute unused_stack cell depending of
	 * the stack size.
	 */
	stack_cell = stack->size / PAGE_SIZE;
	if(stack_cell >= USTACK_TABLE_SIZE)
		stack_cell = USTACK_TABLE_SIZE - 1;

	/*
	 * Destroy elements from special unused_stack cell which were
	 * added to it earlier
	 */
	while(unused_stacks[USTACK_TABLE_SIZE - 1]) {
		xt_stack_t *dead_stack = unused_stacks[USTACK_TABLE_SIZE - 1];
		unused_stacks[USTACK_TABLE_SIZE - 1] = dead_stack->next;
		munmap(dead_stack->base, dead_stack->size);
		free(dead_stack);
	}

	/*
	 * Add this stack into an appropriate cell, depending of
	 * the stack size.
	 */
	stack->next = unused_stacks[stack_cell];
	unused_stacks[stack_cell] = stack;
}
