#ifndef	__XT_TIMER_H__
#define	__XT_TIMER_H__

/*
 * Timer states
 */
typedef enum {
	XTS_STOPPED	= 0,	/* Timer is not running */
	XTS_RUNNING	= 1,	/* Timer is running */
} xt_timer_state;

/*
 * Timer structure
 */
typedef struct xt_timer_s {
	xt_timer_state state;

	void (*callback)(struct xt_timer_s *, void *data);
	void *data;

	int dst_tick;

	struct xt_timer_s *h_prev;
	struct xt_timer_s *h_next;
} xt_timer_t;


/*
 * Set timer to wait specified number of ticks (hz)
 */
void _xt_timer_start(xt_timer_t *timer,
		int wait_ticks,
		void (*callback)(struct xt_timer_s *, void *data),
		void *data);

/*
 * Stop executing timer
 */
void _xt_timer_stop(xt_timer_t *timer);


/*
 * Execute timers.
 * Return the number of HZ to the nearest timer or 0 if there are
 * no timers left.
 * Returns some small value, if timer not found in the near future.
 */
int _xt_timer_work();


/*
 * Sleep for specified number of ticks.
 * If ticks == 0, immediate context switch will be performed.
 * Function returns the number of ticks unslept.
 */
unsigned int _xt_sleep(unsigned int ticks);

/*
 * Schedule a wakeup after specified number of ticks.
 */
void _xt_schedule_wakeup(unsigned int ticks);


/*
 * Initialize some internal timer stuff.
 * Will return -1 on error.
 */
int _xt_timer_initialize(void);

#endif	/* __XT_TIMER_H__ */
